package ovh.herisson.Clyde.EndPoints;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.InscriptionService;
import ovh.herisson.Clyde.Tables.InscriptionRequest;
import ovh.herisson.Clyde.Tables.RequestState;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.User;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")

public class InscriptionController {


    private final InscriptionService inscriptionServ;
    private final AuthenticatorService authServ;

    public InscriptionController(InscriptionService inscriptionServ, AuthenticatorService authServ){
        this.inscriptionServ = inscriptionServ;
        this.authServ = authServ;
    }


    @GetMapping("/requests/register")
    public ResponseEntity<Iterable<Map<String,Object>>> getAllRequests(@RequestHeader("Authorization") String token){

        if (!isSecretaryOrAdmin(token)){return new UnauthorizedResponse<>(null);}

        Iterable<InscriptionRequest> inscriptionRequests = inscriptionServ.getAll();
        ArrayList<Map<String,Object>> toReturn = new ArrayList<>();

        for (InscriptionRequest i:inscriptionRequests){
            toReturn.add(requestWithoutPassword(i));
        }

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }


    @GetMapping("/request/register/{id}")
    public ResponseEntity<Map<String,Object>> getById(@PathVariable long id){
        InscriptionRequest inscriptionRequest = inscriptionServ.getById(id);
        if (inscriptionRequest == null) {return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);}

        return new ResponseEntity<>(requestWithoutPassword(inscriptionRequest), HttpStatus.OK);
    }

    @GetMapping("request/user/{id}")
    public ResponseEntity<InscriptionRequest> getUserInscriptionRequest(@PathVariable long id, @RequestHeader("Authorize") String token){
        //todo return l'inscriptionRequest ACTUELLE du user (check si le poster est bien le même que id target ou secretariat)
        return null;
    }

    @PatchMapping("/request/register/{id}")
    public ResponseEntity<InscriptionRequest> changeRequestState(@PathVariable long id,
                                                                 @RequestHeader("Authorize") String token,
                                                                 @RequestBody RequestState requestState)
    {
        if (!isSecretaryOrAdmin(token)) return new UnauthorizedResponse<>(null);
        inscriptionServ.modifyState(id, requestState);
        return null;
    }

    private Map<String, Object> requestWithoutPassword(InscriptionRequest inscriptionRequest) {
        Map<String, Object> toReturn = new HashMap<>();

        toReturn.put("id", inscriptionRequest.getId());
        toReturn.put("firstName", inscriptionRequest.getFirstName());
        toReturn.put("lastName", inscriptionRequest.getLastName());
        toReturn.put("address", inscriptionRequest.getAddress());
        toReturn.put("birthDate", inscriptionRequest.getBirthDate());
        toReturn.put("country", inscriptionRequest.getCountry());
        toReturn.put("curriculum", inscriptionRequest.getCurriculum());
        toReturn.put("profilePictureUrl", inscriptionRequest.getProfilePicture());
        toReturn.put("state", inscriptionRequest.getState());
        return toReturn;
    }


    private boolean isSecretaryOrAdmin(String authorization){
        if (authorization ==null)
            return false;

        User poster = authServ.getUserFromToken(authorization);
        if (poster == null) return false;

        return poster.getRole() == Role.Secretary && poster.getRole() == Role.Admin;
    }
}